@@ -9,6 +9,8 @@ def distinct(key_mapper=None):
     this operator should be considered carefully due to the maintenance
     of an internal lookup structure which can grow large.
 
+    The source must be a MuxObservable.
+
     .. marble::
         :alt: distinct
 
@@ -21,16 +23,6 @@ def distinct(key_mapper=None):
     implementation does not allow to specify a comparer: The key must be
     hashable.
 
-
-    Examples:
-        >>> data = range(200000, 0, -1)
-        >>> rx.from_(data).pipe(
-        >>>     ops.distinct(),
-        >>> )
-
-    Source:
-        An Observable or a MuxObservable
-
     Args:
         key_mapper: [Optional]  A function to compute the comparison
             key for each element.
@@ -42,43 +34,50 @@ def distinct(key_mapper=None):
         sequence.
     """
     def _distinct(source):
-        mux = True if isinstance(source, rs.MuxObservable) else False
-
         def on_subscribe(observer, scheduler=None):
-            hashset = {} if mux else set()
+            state = None
 
             def on_next(x):
-                if isinstance(x, rs.OnCreateMux):
-                    hashset[x.key] = set()
-                    observer.on_next(x)
-                    return
-                elif isinstance(x, rs.OnCompletedMux) \
-                or isinstance(x, rs.OnErrorMux):
-                    del hashset[x.key]
-                    observer.on_next(x)
-                    return
+                nonlocal state
 
-                i = x.item if mux else x
-                key = i
+                if type(x) is rs.OnNextMux:
+                    i = x.item
+                    key = i
 
-                if key_mapper:
-                    try:
-                        key = key_mapper(i)
-                    except Exception as ex:
-                        observer.on_error(ex)
-                        return
+                    if key_mapper:
+                        try:
+                            key = key_mapper(i)
+                        except Exception as ex:
+                            observer.on_error(ex)
+                            return
 
-                state = hashset[x.key] if mux else hashset
-                if key not in state:
-                    state.add(key)
+                    _state = x.store.get_state(state, x.key)
+                    if key not in _state:
+                        _state.add(key)
+                        observer.on_next(x)
+                elif type(x) is rs.OnCreateMux:
+                    x.store.add_key(state, x.key)
+                    x.store.set_state(state, x.key, set())
                     observer.on_next(x)
+
+                elif type(x) in [rs.OnCompletedMux, rs.OnErrorMux]:
+                    x.store.del_key(state, x.key)
+                    observer.on_next(x)
+
+                elif type(x) is rs.state.ProbeStateTopology:
+                    state = x.topology.create_state(name="distinct", data_type='set')
+                    observer.on_next(x)
+                else:
+                    if state is None:
+                        observer.on_error(ValueError("No state configured in distinct operator. A state store operator is probably missing in the graph"))
+                    observer.on_next(x)
+
             return source.subscribe_(
                 on_next,
                 observer.on_error,
                 observer.on_completed,
                 scheduler)
-        if mux is True:
-            return rs.MuxObservable(on_subscribe)
-        else:
-            return rx.create(on_subscribe)
+
+        return rs.MuxObservable(on_subscribe)
+
     return _distinct
