@@ -1,5 +1,8 @@
+import os
 from collections import namedtuple
+import typing
 import tempfile
+import pytest
 import rx
 import rx.operators as ops
 import rxsci.container.csv as csv
@@ -7,18 +10,21 @@ import rxsci.container.csv as csv
 
 def process(source, pipeline=None):
     actual_data = []
+    actual_error = []
 
     if pipeline is not None:
         source.pipe(*pipeline).subscribe(
             on_next=actual_data.append,
-            on_error=lambda e: print(e)
+            on_error=actual_error.append,
         )
     else:
         source.subscribe(
             on_next=actual_data.append,
-            on_error=lambda e: print(e)
+            on_error=actual_error.append,
         )
 
+    if len(actual_error) > 0:
+        raise actual_error[0]
     return actual_data
 
 
@@ -41,6 +47,54 @@ def test_parser():
     assert actual_data[1] == (7, 'quick', False)
 
 
+def test_parser_typed():
+    class TestLine(typing.NamedTuple):
+        foo: int
+        bar: str
+        buzz: bool
+
+    parser = csv.create_line_parser(dtype=TestLine)
+
+    actual_data = process(rx.from_([
+        "42,the,True",
+        "07,quick,False",
+    ]), [ops.map(parser)])
+
+    assert len(actual_data) == 2
+    assert actual_data[0] == (42, 'the', True)
+    assert actual_data[1] == (7, 'quick', False)
+
+
+def test_parser_empty_numbers():
+    parser = csv.create_line_parser(
+        dtype=[
+            ("foo", "int"),
+            ("bar", "float"),
+        ]
+    )
+
+    actual_data = process(rx.from_([
+        ",",
+    ]), [ops.map(parser)])
+
+    assert len(actual_data) == 1
+    assert actual_data[0] == (None, None)
+
+
+def test_parser_invalid_numbers():
+    parser = csv.create_line_parser(
+        dtype=[
+            ("foo", "int"),
+            ("bar", "float"),
+        ]
+    )
+
+    with pytest.raises(ValueError):
+        process(rx.from_([
+            "as,ds",
+        ]), [ops.map(parser)])
+
+
 def test_load():
     parser = csv.create_line_parser(
         dtype=[
@@ -75,13 +129,19 @@ def test_load_quoted():
         '2,"\\"brown fox\\""',
         '3,"a\"$#ܟ<a;.b^F ^M^E^Aa^Bov^D^\"[^BƆm^A^Q^]#lx"',
         '4,""',
+        '5,"\\"a\\",b"',
+        '6,",ab"',
+        '7,","',
     ]), [csv.load(parser)])
 
-    assert len(actual_data) == 4
+    assert len(actual_data) == 7
     assert actual_data[0] == (1, 'the, quick')
     assert actual_data[1] == (2, '"brown fox"')
     assert actual_data[2] == (3, 'a"$#ܟ<a;.b^F ^M^E^Aa^Bov^D^"[^BƆm^A^Q^]#lx')
     assert actual_data[3] == (4, '')
+    assert actual_data[4] == (5, '"a",b')
+    assert actual_data[5] == (6, ',ab')
+    assert actual_data[6] == (7, ',')
 
 
 def test_load_error():
@@ -132,6 +192,7 @@ def test_load_ignore_error():
         error = e
 
     source = rx.from_([
+        "a,the,True",
         "42,the,True",
         "07",
     ])
@@ -154,14 +215,17 @@ def test_load_from_file():
         ]
     )
 
-    with tempfile.NamedTemporaryFile(mode='w') as f:
-        f.write("foo,bar,buzz\n")
-        f.write("42,the,True\n")
-        f.write("07,quick,False\n")
-        f.flush()
+    with tempfile.TemporaryDirectory() as d:
+        f_name = os.path.join(d, "test.csv")
+
+        with open(f_name, mode="w") as f:
+            f.write("foo,bar,buzz\n")
+            f.write("42,the,True\n")
+            f.write("07,quick,False")
+            f.flush()
 
         actual_data = process(
-            csv.load_from_file(f.name, parser),
+            csv.load_from_file(f_name, parser),
             None,
         )
 
@@ -198,6 +262,34 @@ def test_dump():
     assert actual_data == expected_data
 
 
+def test_dump_to_file():
+    x = namedtuple('x', ['foo', 'bar', 'buz'])
+    source = [
+        x(foo='a', bar=1, buz=True),
+        x(foo='b', bar=2, buz=False),
+        x(foo='ab', bar=42, buz=False),
+        x(foo='cl', bar=None, buz=False),
+        x(foo='cl', bar=[1, '2', "3"], buz=False),
+    ]
+    expected_data = \
+        'foo,bar,buz\n' \
+        '"a",1,True\n' \
+        '"b",2,False\n' \
+        '"ab",42,False\n' \
+        '"cl",,False\n' \
+        '"cl","[1, \'2\', \'3\']",False\n'
+
+    with tempfile.TemporaryDirectory() as d:
+        f = os.path.join(d, "test.csv")
+        rx.from_(source).pipe(
+            csv.dump_to_file(f, encoding='utf-8'),
+        ).subscribe()
+
+        with open(f, 'r') as f1:
+            actual_data = f1.read()
+    assert actual_data == expected_data
+
+
 def test_dump_with_quote():
     x = namedtuple('x', ['foo', 'bar', 'buz'])
     source = [
