@@ -33,11 +33,14 @@ def test_to_list_mux():
 
     rx.from_(source).pipe(
         rs.cast_as_mux_observable(),
-        rs.data.to_list(),
+        rs.state.with_memory_store(
+            rs.data.to_list(),
+        ),
     ).subscribe(
         on_next=actual_result.append
     )
 
+    actual_result = [r._replace(store=None) for r in actual_result]
     assert actual_result == [
         rs.OnCreateMux((1,)),
         rs.OnCreateMux((2,)),
